#------------------------------------------------------------------------------------------------
# Permission to use, copy, modify, and distribute this software for 
# any purpose and without fee is hereby granted.
#
# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#------------------------------------------------------------------------------------------------
#    Name:	ARCH_GEN
#      By:	sdmitch
#   Usage:	Generates a Arched enity given specs entered
#	 Note:	Since the radius and chord are inter-dependent, only one may be entered and the other
#			is calculated.
#    Date:	Jul 2011
#------------------------------------------------------------------------------------------------
#
require 'Sketchup'
#
filename=File.basename(__FILE__)
#
if not file_loaded?(filename)
    UI.menu('Plugins').add_item('Arch Gen') { Arch_Maker.arch_gen }
end
#
file_loaded(filename)
#
module Arch_Maker

	def Arch_Maker.arch_gen
		prompts = ["Chord:","Height:","Radius:","Width:","Depth:"]
		defaults = [44.25.inch,7.25.inch,37.77.inch,8.inch, 4.inch]
		ans=UI.inputbox(prompts,defaults,"Arch Specifications")
		if !ans then
			puts "Inputbox canceled" if $sdm_debug
			return
		end
		mod=Sketchup.active_model;ent=mod.entities
		mod.start_operation "Generate Arch"
		chord,bulge,n_rad,width,depth=ans
		if chord == 0.0 && n_rad == 0.0 then
			UI.messagebox "You must enter a Chord length or Radius"
			return
		elsif chord != 0.0 && n_rad != 00 then
			UI.messagebox "Enter either a Chord or a Radius but not BOTH"
			return
		end
		# calculate radius from chord and bulge if n_rad = 0
		n_rad = Arch_Maker.calc_radius(chord,bulge) if n_rad == 0.0; 
		puts "n_rad=#{n_rad}" if $sdm_debug
		dist=n_rad-bulge; r_edges=[];l_edges=[]; edges=[]
		ctr=Geom::Point3d.new(0,0,0); xa=[1,0,0]; ya=[0,1,0]; za=[0,0,1]
	#  --------------------------------------------------------------------  #
		if chord == 0.0
			p1=ctr.offset(ya,dist).offset(xa,1)
			p2=ctr.offset(ya,dist).offset(xa,-1)
			pts=Arch_Maker.line_sphere_intersection(p2,p1,ctr,n_rad); p1=pts[0]; p2=pts[1]
			puts "p1=#{p1}, p2=#{p2}" if $sdm_debug
		else
			puts "chord=#{chord}" if $sdm_debug
			p1=ctr.offset(ya,dist).offset(xa,chord/2.0)
			p2=ctr.offset(ya,dist).offset(xa,-chord/2.0)
			puts "p1=#{p1}, p2=#{p2}" if $sdm_debug
		end
	#  --------------------------------------------------------------------  #
		sv1 = ctr.vector_to(p1).normalize; puts "sv1=",sv1 if $sdm_debug
		sv2 = ctr.vector_to(p2).normalize; puts "sv2=",sv2 if $sdm_debug
		ang = sv1.angle_between(sv2); puts "ang=#{ang}" if $sdm_debug
		n_arc=ent.add_arc(ctr,sv1,za,n_rad,0,ang,30); 
		puts "n_arc length=#{n_arc.length}" if $sdm_debug 
	#  --------------------------------------------------------------------  #
		p3=p1.offset(xa,8.0); p4=p2.offset(xa,-8.0)
		puts "p3=#{p3}, p4=#{p4}" if $sdm_debug
		r_edges[1]=ent.add_line(p1,p3); l_edges[0]=ent.add_line(p2,p4)
		p5=p3.offset(ya); p6=p4.offset(ya); o_rad=n_rad+width
		pts=Arch_Maker.line_sphere_intersection(p3,p5,ctr,o_rad); p5=pts[0]
		pts=Arch_Maker.line_sphere_intersection(p4,p6,ctr,o_rad); p6=pts[0]
		puts "p5=#{p5}, p6=#{p6}, o_rad=#{o_rad}" if $sdm_debug
		r_edges[0]=ent.add_line(p3,p5); l_edges[1]=ent.add_line(p4,p6)
	#  --------------------------------------------------------------------  #
		sv1 = ctr.vector_to(p5).normalize; puts "sv1=",sv1 if $sdm_debug
		sv2 = ctr.vector_to(p6).normalize; puts "sv2=",sv2 if $sdm_debug
		ang = sv1.angle_between(sv2); puts "ang=#{ang}" if $sdm_debug
		o_arc=ent.add_arc(ctr,sv1,za,o_rad,0,ang,30)
		puts "o_arc length=#{o_arc.length}" if $sdm_debug 
	#  --------------------------------------------------------------------  #
		edges = n_arc + l_edges + o_arc.reverse + r_edges
		puts "edges length=#{edges.length}" if $sdm_debug 
		face=ent.add_face(edges);  puts "Face failed" if !face
		face.pushpull -depth if face
	#  --------------------------------------------------------------------  #
		mod.commit_operation
	end
	
	def Arch_Maker.line_sphere_intersection(pt0,pt1,ctr,ros)
		dx = pt1.x - pt0.x
		dy = pt1.y - pt0.y
		dz = pt1.z - pt0.z
		a =  dx**2 + dy**2 + dz**2
		b =  2*( dx*(pt0.x - ctr.x) + dy*(pt0.y - ctr.y) + dz*(pt0.z - ctr.z))
		c =  ctr.x**2 + ctr.y**2 + ctr.z**2 + pt0.x**2 + pt0.y**2 + pt0.z**2 - 2*( ctr.x*pt0.x + ctr.y*pt0.y + ctr.z*pt0.z ) - ros**2
		i =   b * b - 4 * a * c ; p=[]
		# first intersection
		mu = (-b + Math.sqrt( (b**2) - 4*a*c )) / (2*a);
		x = pt0.x + mu*dx;
		y = pt0.y + mu*dy;
		z = pt0.z + mu*dz;
		p[0] = Geom::Point3d.new(x,y,z)
		# second intersection
		mu = (-b - Math.sqrt((b**2) - 4*a*c )) / (2*a);
		x = pt0.x + mu*dx;
		y = pt0.y + mu*dy;
		z = pt0.z + mu*dz;
		p[1] = Geom::Point3d.new(x,y,z)
		return p;
	end
	
	def Arch_Maker.calc_radius(c,b)
		p1 = Geom::Point3d.new(-c/2.0,0,0)
		p2 = Geom::Point3d.new(c/2.0,0,0)
		p3 = Geom::Point3d.new(0,b,0)
		v1 = p1.vector_to(p3);v2=p2.vector_to(p3)
		d1 = p1.distance(p3); d2 = p2.distance(p3)
		mp1 = p1.offset(v1,d1/2.0)
		mp2 = p2.offset(v2,d2/2.0)
		v1 = Geom::Vector3d.new(-v1.y,v1.x,0); 
		v2 = Geom::Vector3d.new(-v2.y,v2.x,0)
		ln1=[mp1,v1]; ln2=[mp2,v2]
		poi=Geom.intersect_line_line ln1,ln2
		r=poi.distance(p3)
	end
		
end
